/********************************** (C) COPYRIGHT *******************************
* File Name          : GATTprofile.h
* Author             : WCH
* Version            : V1.0
* Date               : 2018/12/11
* Description        : 
            
*******************************************************************************/

#ifndef LIGHTPROFILE_H
#define LIGHTPROFILE_H

#ifdef __cplusplus
extern "C"
{
#endif

/*********************************************************************
 * INCLUDES
 */

/*********************************************************************
 * CONSTANTS
 */

// Profile Parameters
#define lightProfile_CHAR1                   0  // RW uint8 - Profile Characteristic 1 value
#define lightProfile_CHAR2                   1  // RW uint8 - Profile Characteristic 2 value
#define lightProfile_CHAR3                   2  // RW uint8 - Profile Characteristic 3 value

  
// Simple Profile Service UUID
#define lightProfile_SERV_UUID               0xFDE0
    
// Key Pressed UUID
#define lightProfile_CHAR1_UUID            0xFDE1
#define lightProfile_CHAR2_UUID            0xFDE2
#define lightProfile_CHAR3_UUID            0xFDE3

  
// Simple Keys Profile Services bit fields
#define lightProfile_SERVICE               0x00000001
extern const char lightReadbuf1[5];
extern const char lightReadbuf2[5];
/*********************************************************************
 * TYPEDEFS
 */

  
/*********************************************************************
 * MACROS
 */

/*********************************************************************
 * Profile Callbacks
 */



// Callback when a characteristic value has changed
typedef void (*lightProfileChange_t)( uint8 paramID );

typedef struct
{
  lightProfileChange_t        pfnlightProfileChange;  // Called when characteristic value changes
} lightProfileCBs_t;



/*********************************************************************
 * API FUNCTIONS 
 */


/*
 * lightProfile_AddService- Initializes the Simple GATT Profile service by registering
 *          GATT attributes with the GATT server.
 *
 * @param   services - services to add. This is a bit map and can
 *                     contain more than one service.
 */

extern bStatus_t lightProfile_AddService( uint32 services );

/*
 * lightProfile_RegisterAppCBs - Registers the application callback function.
 *                    Only call this function once.
 *
 *    appCallbacks - pointer to application callbacks.
 */
extern bStatus_t lightProfile_RegisterAppCBs( lightProfileCBs_t *appCallbacks );


bStatus_t lightProfile_Notify(uint16 connHandle, attHandleValueNoti_t *pNoti);

/*********************************************************************
*********************************************************************/

#ifdef __cplusplus
}
#endif

#endif
