/********************************** (C) COPYRIGHT *******************************
 * File Name          : lightprofile.C
 * Author             : WCH
 * Version            : V1.0
 * Date               : 2021/07/08
 * Description        :

 *******************************************************************************/

/*********************************************************************
 * INCLUDES
 */
#include "CONFIG.h"
#include "lightprofile.h"
#include "CH57x_common.h"
#include "light.h"
/*********************************************************************
 * MACROS
 */
// Position of simpleProfilechar3 value in attribute array
#define LIGHTPROFILE_CHAR3_VALUE_POS            8
/*********************************************************************
 * CONSTANTS
 */

#define SERVAPP_NUM_ATTR_SUPPORTED        11

/*********************************************************************
 * TYPEDEFS
 */

/*********************************************************************
 * GLOBAL VARIABLES
 */
// Simple GATT Profile Service UUID: 0xFFF0
CONST uint8 lightProfileServUUID[ATT_BT_UUID_SIZE] = { LO_UINT16(
        lightProfile_SERV_UUID), HI_UINT16(lightProfile_SERV_UUID) };

// Characteristic 1 UUID: 0xFFF1
CONST uint8 lightProfilechar1UUID[ATT_BT_UUID_SIZE] = { LO_UINT16(
        lightProfile_CHAR1_UUID), HI_UINT16(lightProfile_CHAR1_UUID) };

// Characteristic 2 UUID: 0xFFF2
CONST uint8 lightProfilechar2UUID[ATT_BT_UUID_SIZE] = { LO_UINT16(
        lightProfile_CHAR2_UUID), HI_UINT16(lightProfile_CHAR2_UUID) };

// Characteristic 3 UUID: 0xFFF3
CONST uint8 lightProfilechar3UUID[ATT_BT_UUID_SIZE] = { LO_UINT16(
        lightProfile_CHAR3_UUID), HI_UINT16(lightProfile_CHAR3_UUID) };

/*********************************************************************
 * EXTERNAL VARIABLES
 */

/*********************************************************************
 * EXTERNAL FUNCTIONS
 */

/*********************************************************************
 * LOCAL VARIABLES
 */
const char lightReadbuf1[5] = { "STA:" };
const char lightReadbuf2[5] = { "VAL:" };
static lightProfileCBs_t *lightProfile_AppCBs = NULL;


static uint8 lightProfileChar1WriteBuf[20];

static uint8 lightProfileChar2WriteBuf[20];
/*********************************************************************
 * Profile Attributes - variables
 */

// Simple Profile Service attribute
static CONST gattAttrType_t lightProfileService = { ATT_BT_UUID_SIZE,
        lightProfileServUUID };

// Simple Profile Characteristic 1 Properties
static uint8 lightProfileChar1Props = GATT_PROP_WRITE|GATT_PROP_READ;

// Characteristic 1 Value
static uint8 lightProfileChar1 = 0;

// Simple Profile Characteristic 1 User Description
static uint8 lightProfileChar1UserDesp[20] = "lightSwitch Control\0";

// Simple Profile Characteristic 2 Properties
static uint8 lightProfileChar2Props = GATT_PROP_WRITE|GATT_PROP_READ;

// Characteristic 2 Value
static uint8 lightProfileChar2 = 0;

// Simple Profile Characteristic 2 User Description
static uint8 lightProfileChar2UserDesp[23] = "lightIntensity Control\0";

// Simple Profile Characteristic 3 Properties
static uint8 lightProfileChar3Props = GATT_PROP_NOTIFY;

// Characteristic 3 Value
static uint8 lightProfileChar3 = 0;

// Simple Profile Characteristic 3 User Description
static uint8 lightProfileChar3UserDesp[19] = "light State Notice\0";
// Simple Profile Characteristic 4 Configuration Each client has its own
// instantiation of the Client Characteristic Configuration. Reads of the
// Client Characteristic Configuration only shows the configuration for
// that client and writes only affect the configuration of that client.
static gattCharCfg_t lightProfileChar3Config[4];

/*********************************************************************
 * Profile Attributes - Table
 */

static gattAttribute_t lightProfileAttrTbl[11] =
        {
        // Simple Profile Service
                { { ATT_BT_UUID_SIZE, primaryServiceUUID }, /* type */
                GATT_PERMIT_READ, /* permissions */
                0, /* handle */
                (uint8 *) &lightProfileService /* pValue */
                },

                // Characteristic 1 Declaration
                { { ATT_BT_UUID_SIZE, characterUUID },
                GATT_PERMIT_READ, 0, &lightProfileChar1Props },

                // Characteristic Value 1
                { { ATT_BT_UUID_SIZE, lightProfilechar1UUID },
                        GATT_PERMIT_READ|GATT_PERMIT_WRITE, 0, &lightProfileChar1 },

                // Characteristic 1 User Description
                { { ATT_BT_UUID_SIZE, charUserDescUUID },
                GATT_PERMIT_READ, 0, lightProfileChar1UserDesp },

                // Characteristic 2 Declaration
                { { ATT_BT_UUID_SIZE, characterUUID },
                GATT_PERMIT_READ, 0, &lightProfileChar2Props },

                // Characteristic Value 2
                { { ATT_BT_UUID_SIZE, lightProfilechar2UUID },
                        GATT_PERMIT_READ|GATT_PERMIT_WRITE, 0, &lightProfileChar2 },

                // Characteristic 2 User Description
                { { ATT_BT_UUID_SIZE, charUserDescUUID },
                GATT_PERMIT_READ, 0, lightProfileChar2UserDesp },

                // Characteristic 3 Declaration
                { { ATT_BT_UUID_SIZE, characterUUID },
                GATT_PERMIT_READ, 0, &lightProfileChar3Props },
                // Characteristic Value 3
                { { ATT_BT_UUID_SIZE, lightProfilechar3UUID }, 0, 0,
                        &lightProfileChar3 },

                // Characteristic 3 configuration
                { { ATT_BT_UUID_SIZE, clientCharCfgUUID },
                GATT_PERMIT_READ | GATT_PERMIT_WRITE, 0,
                        (uint8 *) lightProfileChar3Config },

                // Characteristic 3 User Description
                { { ATT_BT_UUID_SIZE, charUserDescUUID },
                GATT_PERMIT_READ, 0, lightProfileChar3UserDesp }, };

/*********************************************************************
 * LOCAL FUNCTIONS
 */
static bStatus_t lightProfile_ReadAttrCB(uint16 connHandle,
        gattAttribute_t *pAttr, uint8 *pValue, uint16 *pLen, uint16 offset,
        uint16 maxLen, uint8 method);
static bStatus_t lightProfile_WriteAttrCB(uint16 connHandle,
        gattAttribute_t *pAttr, uint8 *pValue, uint16 len, uint16 offset,
        uint8 method);

static void lightProfile_HandleConnStatusCB(uint16 connHandle, uint8 changeType);

/*********************************************************************
 * PROFILE CALLBACKS
 */
// Simple Profile Service Callbacks
gattServiceCBs_t lightProfileCBs = { lightProfile_ReadAttrCB, // Read callback function pointer
        lightProfile_WriteAttrCB, // Write callback function pointer
        NULL                       // Authorization callback function pointer
        };

/*********************************************************************
 * PUBLIC FUNCTIONS
 */

/*********************************************************************
 * @fn      lightProfile_AddService
 *
 * @brief   Initializes the Simple Profile service by registering
 *          GATT attributes with the GATT server.
 *
 * @param   services - services to add. This is a bit map and can
 *                     contain more than one service.
 *
 * @return  Success or Failure
 */
bStatus_t lightProfile_AddService(uint32 services) {
    uint8 status = SUCCESS;
    // Initialize Client Characteristic Configuration attributes
    GATTServApp_InitCharCfg( INVALID_CONNHANDLE, lightProfileChar3Config );
    // Register with Link DB to receive link status change callback
    linkDB_Register( lightProfile_HandleConnStatusCB );
    if (services & lightProfile_SERVICE) {
        // Register GATT attribute list and CBs with GATT Server App
        status = GATTServApp_RegisterService(lightProfileAttrTbl,
                GATT_NUM_ATTRS(lightProfileAttrTbl),
                GATT_MAX_ENCRYPT_KEY_SIZE, &lightProfileCBs);
    }

    return (status);
}

/*********************************************************************
 * @fn      lightProfile_RegisterAppCBs
 *
 * @brief   Registers the application callback function. Only call 
 *          this function once.
 *
 * @param   callbacks - pointer to application callbacks.
 *
 * @return  SUCCESS or bleAlreadyInRequestedMode
 */
bStatus_t lightProfile_RegisterAppCBs(lightProfileCBs_t *appCallbacks) {
    if (appCallbacks) {
        lightProfile_AppCBs = appCallbacks;

        return ( SUCCESS);
    } else {
        return ( bleAlreadyInRequestedMode);
    }
}

/*********************************************************************
 * @fn          lightProfile_ReadAttrCB
 *
 * @brief       Read an attribute.
 *
 * @param       connHandle - connection message was received on
 * @param       pAttr - pointer to attribute
 * @param       pValue - pointer to data to be read
 * @param       pLen - length of data to be read
 * @param       offset - offset of the first octet to be read
 * @param       maxLen - maximum length of data to be read
 *
 * @return      Success or Failure
 */
static bStatus_t lightProfile_ReadAttrCB(uint16 connHandle,
        gattAttribute_t *pAttr, uint8 *pValue, uint16 *pLen, uint16 offset,
        uint16 maxLen, uint8 method) {
    bStatus_t status = SUCCESS;
    if (pAttr->type.len == ATT_BT_UUID_SIZE) {
        // 16-bit UUID
        uint16 uuid = BUILD_UINT16(pAttr->type.uuid[0], pAttr->type.uuid[1]);
        switch (uuid) {
        case lightProfile_CHAR1_UUID: {
            *pLen = 5;
            sprintf(pValue, "%s%1d", lightReadbuf1, lightSwitchState);
            break;
        }
        case lightProfile_CHAR2_UUID:
            *pLen = 7;
            sprintf(pValue, "%s%3d", lightReadbuf2, lightIntensityValue);
            break;
        case lightProfile_CHAR3_UUID:
            break;

        default:
            // Should never get here! (characteristics 3 and 4 do not have read permissions)
            *pLen = 0;
            status = ATT_ERR_ATTR_NOT_FOUND;
            break;
        }
    } else {
        // 128-bit UUID
        *pLen = 0;
        status = ATT_ERR_INVALID_HANDLE;
    }

    return (status);
}

/*********************************************************************
 * @fn      lightProfile_WriteAttrCB
 *
 * @brief   Validate attribute data prior to a write operation
 *
 * @param   connHandle - connection message was received on
 * @param   pAttr - pointer to attribute
 * @param   pValue - pointer to data to be written
 * @param   len - length of data
 * @param   offset - offset of the first octet to be written
 *
 * @return  Success or Failure
 */
static bStatus_t lightProfile_WriteAttrCB(uint16 connHandle,
        gattAttribute_t *pAttr, uint8 *pValue, uint16 len, uint16 offset,
        uint8 method) {
    bStatus_t status = SUCCESS;
    uint8 notifyApp = 0xFF;

    // If attribute permissions require authorization to write, return error
    if (gattPermitAuthorWrite(pAttr->permissions)) {
        // Insufficient authorization
        return ( ATT_ERR_INSUFFICIENT_AUTHOR);
    }

    if (pAttr->type.len == ATT_BT_UUID_SIZE) {
        // 16-bit UUID
        uint16 uuid = BUILD_UINT16(pAttr->type.uuid[0], pAttr->type.uuid[1]);
        switch (uuid) {
        case lightProfile_CHAR1_UUID: {
            uint8* p_rec_buf;
            uint8 ledstate;
            p_rec_buf = pValue;
            memcpy(lightProfileChar1WriteBuf,p_rec_buf,len);
            if ((len == 5) && (lightProfileChar1WriteBuf[0] == 'S') && (lightProfileChar1WriteBuf[1] == 'T')
                    && (lightProfileChar1WriteBuf[2] == 'A') && (lightProfileChar1WriteBuf[3]== ':')) {
                ledstate = lightProfileChar1WriteBuf[4] - '0';
                lightSwitch(ledstate);
                notifyApp = lightProfile_CHAR1;
            }
            break;
        }
        case lightProfile_CHAR2_UUID: {
            uint8* p_rec_buf;
            uint8 ledvalue;
            p_rec_buf = pValue;
            memcpy(lightProfileChar2WriteBuf,p_rec_buf,len);
            if ((len == 7) && (lightProfileChar2WriteBuf[0] == 'V') && (lightProfileChar2WriteBuf[1] == 'A')
                    && (lightProfileChar2WriteBuf[2]== 'L') && (lightProfileChar2WriteBuf[3] == ':')) {
                ledvalue = (lightProfileChar2WriteBuf[4] - '0') * 100
                        + (lightProfileChar2WriteBuf[5] - '0') * 10
                        + (lightProfileChar2WriteBuf[6] - '0');
                lightIntensityControl(ledvalue);
                notifyApp = lightProfile_CHAR2;
            }
            break;
        }
        case GATT_CLIENT_CHAR_CFG_UUID:
          status = GATTServApp_ProcessCCCWriteReq( connHandle, pAttr, pValue, len,
                                                   offset, GATT_CLIENT_CFG_NOTIFY );
          notifyApp = lightProfile_CHAR1;
          break;
        default:
            // Should never get here! (characteristics 2 and 4 do not have write permissions)
            status = ATT_ERR_ATTR_NOT_FOUND;
            break;
        }
    } else {
        // 128-bit UUID
        status = ATT_ERR_INVALID_HANDLE;
    }

    // If a charactersitic value changed then callback function to notify application of change
    if ((notifyApp != 0xFF) && lightProfile_AppCBs
            && lightProfile_AppCBs->pfnlightProfileChange) {
        lightProfile_AppCBs->pfnlightProfileChange(notifyApp);
    }

    return (status);
}

/*********************************************************************
 * @fn          simpleProfile_Notify
 *
 * @brief       Send a notification containing a heart rate
 *              measurement.
 *
 * @param       connHandle - connection handle
 * @param       pNoti - pointer to notification structure
 *
 * @return      Success or Failure
 */
bStatus_t lightProfile_Notify(uint16 connHandle, attHandleValueNoti_t *pNoti) {
    uint16 value = GATTServApp_ReadCharCfg(connHandle, lightProfileChar3Config);

    // If notifications enabled
    if (value & GATT_CLIENT_CFG_NOTIFY) {
        // Set the handle
        pNoti->handle =
                lightProfileAttrTbl[LIGHTPROFILE_CHAR3_VALUE_POS].handle;

        // Send the notification
        return GATT_Notification(connHandle, pNoti, FALSE);
    }
    return bleIncorrectMode;
}


/*********************************************************************
 * @fn          simpleProfile_HandleConnStatusCB
 *
 * @brief       Simple Profile link status change handler function.
 *
 * @param       connHandle - connection handle
 * @param       changeType - type of change
 *
 * @return      none
 */
static void lightProfile_HandleConnStatusCB( uint16 connHandle, uint8 changeType )
{
  // Make sure this is not loopback connection
  if ( connHandle != LOOPBACK_CONNHANDLE )
  {
    // Reset Client Char Config if connection has dropped
    if ( ( changeType == LINKDB_STATUS_UPDATE_REMOVED )      ||
         ( ( changeType == LINKDB_STATUS_UPDATE_STATEFLAGS ) &&
           ( !linkDB_Up( connHandle ) ) ) )
    {
      GATTServApp_InitCharCfg( connHandle, lightProfileChar3Config );
    }
  }
}

/*********************************************************************
 *********************************************************************/
